/**
 * Main Controller
 * @method  controller
 * @param   {object}   args
 */
(function controller(args) {
  'use strict';

  $.index.open();

  $.addListener($.native, 'click', nativeMethod);
  $.addListener($.filesystem, 'click', filesystemMethod);
  $.addListener($.mixedA, 'click', mixedMethodA);
  $.addListener($.mixedB, 'click', mixedMethodB);

  /**
   * Uses pure Android native methods to check and request permissions
   * @method  nativeMethod
   * @param   {object}     e
   */
  function nativeMethod(e) {
    Ti.API.debug('Native method:');
    Ti.API.debug('Check permissions with: Ti.Android.hasPermission');
    Ti.API.debug('Requesting permissions with: Ti.Android.requestPermissions');
    Ti.API.debug('Starting...');

    if (!hasStoragePermission()) {
      requestPermission('android.permission.WRITE_EXTERNAL_STORAGE');
    }
  }

  /**
   * Uses Ti.Android.hasPermissios to check if a permission has been granted
   * @method  hasStoragePermission
   * @returns {Boolean}
   */
  function hasStoragePermission() {
    Ti.API.debug('hasStoragePermission');

    var hasPermission = Ti.Android.hasPermission('android.permission.WRITE_EXTERNAL_STORAGE');

    Ti.API.debug('Ti.Android.hasPermission("android.permission.WRITE_EXTERNAL_STORAGE"): ' + hasPermission);

    return hasPermission;
  }

  /**
   * Uses ti.Android.requestPermissions to request a permission
   * @method  requestPermission
   * @param   {string}          permission
   */
  function requestPermission(permission) {
    Ti.API.debug('requestPermission: ' + permission);
    Ti.API.debug('Ti.Android.requestPermissions: ' + permission);

    Ti.Android.requestPermissions(permission, requestPermissionCallback.bind(null, permission));
  }

  /**
   * Uses Filesystem to check and request permissions for storage
   * @method  filesystemMethod
   * @param   {object}         e
   */
  function filesystemMethod(e) {
    Ti.API.debug('Filesystem method:');
    Ti.API.debug('Check permissions with: Ti.Filesystem.hasStoragePermissions');
    Ti.API.debug('Requesting permissions with: Ti.Filesystem.requestStoragePermissions');
    Ti.API.debug('Starting...');

    if (!hasFilesystemStoragePermission()) {
      requestFilesystemPermission();
    }
  }

  /**
   * Uses Ti.Filesystem.hasStoragePermissions to check if a storage permission
   * has been granted
   * @method  hasFilesystemStoragePermission
   * @returns {Boolean}
   */
  function hasFilesystemStoragePermission() {
    Ti.API.debug('hasFilesystemStoragePermission');

    var hasPermission = Ti.Filesystem.hasStoragePermissions();

    Ti.API.debug('Ti.Filesystem.hasStoragePermissions(): ' + hasPermission);

    return hasPermission;
  }

  /**
   * Uses Ti.Filesystem.requestStoragePermissions to request storage permission
   * @method  requestFilesystemPermission
   */
  function requestFilesystemPermission() {
    Ti.API.debug('requestFilesystemPermission');
    Ti.API.debug('Ti.Filesystem.requestStoragePermissions');

    Ti.Filesystem.requestStoragePermissions(requestPermissionCallback.bind(null, 'Filesystem permission'));
  }

  /**
   * Uses Ti.Android.hasPermission and Ti.Filesystem.requestStoragePermissions
   * @method  mixedMethodA
   * @param   {object}     e
   */
  function mixedMethodA(e) {
    Ti.API.debug('Mixed method A:');
    Ti.API.debug('Check permissions with: Ti.Android.hasPermission');
    Ti.API.debug('Requesting permissions with: Ti.Filesystem.requestStoragePermissions');
    Ti.API.debug('Starting...');

    if (!hasStoragePermission()) {
      requestFilesystemPermission();
    }
  }

  /**
   * Uses Ti.Filesystem.hasStoragePermissions and Ti.Android.requestPermissions
   * @method  mixedMethodB
   * @param   {object}     e
   */
  function mixedMethodB(e) {
    Ti.API.debug('Mixed method B:');
    Ti.API.debug('Check permissions with: Ti.Filesystem.hasStoragePermissions');
    Ti.API.debug('Requesting permissions with: Ti.Android.requestPermissions');
    Ti.API.debug('Starting...');

    if (!hasFilesystemStoragePermission()) {
      requestPermission('android.permission.WRITE_EXTERNAL_STORAGE');
    }
  }

  /**
   * Request permission callback
   * @method  requestPermissionCallback
   * @param   {string}                  permission Permission requested
   * @param   {object}                  e
   */
  function requestPermissionCallback(permission, e) {
    Ti.API.debug('requestPermissionCallback');

    if (e.success) {
      Ti.API.debug('Permission ' + permission + ' granted.');
    } else {
      Ti.API.debug('Permission ' + permission + ' rejected.');
    }

    Ti.API.debug('Finished');
  }

})(arguments[0] |  {});
